@@ -17,6 +17,10 @@
  * USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <stdio.h>
 
@@ -337,17 +341,33 @@ bool CMsgReader::readExtendedClipboard(rdr::S32 len)
 
       lengths[num] = zis.readU32();
 
-      if (!zis.hasData(lengths[num]))
-        throw Exception("Extended clipboard decode error");
-
       if (lengths[num] > (size_t)maxCutText) {
         vlog.error("Extended clipboard data too long (%d bytes) - ignoring",
                    (unsigned)lengths[num]);
-        zis.skip(lengths[num]);
+
+        // Slowly (safely) drain away the data
+        while (lengths[num] > 0) {
+          size_t chunk;
+
+          if (!zis.hasData(1))
+            throw Exception("Extended clipboard decode error");
+
+          chunk = zis.avail();
+          if (chunk > lengths[num])
+            chunk = lengths[num];
+
+          zis.skip(chunk);
+          lengths[num] -= chunk;
+        }
+
         flags &= ~(1 << i);
+
         continue;
       }
 
+      if (!zis.hasData(lengths[num]))
+        throw Exception("Extended clipboard decode error");
+
       buffers[num] = new rdr::U8[lengths[num]];
       zis.readBytes(buffers[num], lengths[num]);
       num++;
@@ -632,11 +652,11 @@ bool CMsgReader::readSetVMwareCursor(int width, int height, const Point& hotspot
   if (!is->hasData(1 + 1))
     return false;
 
+  is->setRestorePoint();
+
   type = is->readU8();
   is->skip(1);
 
-  is->setRestorePoint();
-
   if (type == 0) {
     int len = width * height * (handler->server.pf().bpp/8);
     rdr::U8Array andMask(len);
