@@ -20,6 +20,10 @@
 
 // -=- Configuration.cxx
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
