@@ -32,13 +32,16 @@
 #ifdef HAVE_GNUTLS
 #include <rfb/CSecurityTLS.h>
 #endif
+#ifdef HAVE_NETTLE
+#include <rfb/CSecurityRSAAES.h>
+#endif
 
 using namespace rdr;
 using namespace rfb;
 
 UserPasswdGetter *CSecurity::upg = NULL;
-#ifdef HAVE_GNUTLS
-UserMsgBox *CSecurityTLS::msg = NULL;
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
+UserMsgBox *CSecurity::msg = NULL;
 #endif
 
 StringParameter SecurityClient::secTypes
@@ -46,20 +49,25 @@ StringParameter SecurityClient::secTypes
  "Specify which security scheme to use (None, VncAuth, Plain"
 #ifdef HAVE_GNUTLS
  ", TLSNone, TLSVnc, TLSPlain, X509None, X509Vnc, X509Plain"
+#endif
+#ifdef HAVE_NETTLE
+ ", RA2, RA2ne, RA2_256, RA2ne_256"
 #endif
  ")",
 #ifdef HAVE_GNUTLS
- "X509Plain,TLSPlain,X509Vnc,TLSVnc,X509None,TLSNone,VncAuth,None",
-#else
- "VncAuth,None",
+ "X509Plain,TLSPlain,X509Vnc,TLSVnc,X509None,TLSNone,"
 #endif
+#ifdef HAVE_NETTLE
+ "RA2,RA2_256,RA2ne,RA2ne_256,"
+#endif
+ "VncAuth,None",
 ConfViewer);
 
 CSecurity* SecurityClient::GetCSecurity(CConnection* cc, U32 secType)
 {
   assert (CSecurity::upg != NULL); /* (upg == NULL) means bug in the viewer */
-#ifdef HAVE_GNUTLS
-  assert (CSecurityTLS::msg != NULL);
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
+  assert (CSecurity::msg != NULL);
 #endif
 
   if (!IsSupported(secType))
@@ -73,32 +81,36 @@ CSecurity* SecurityClient::GetCSecurity(CConnection* cc, U32 secType)
 #ifdef HAVE_GNUTLS
   case secTypeTLSNone:
     return new CSecurityStack(cc, secTypeTLSNone,
-                              "TLS with no password",
                               new CSecurityTLS(cc, true));
   case secTypeTLSVnc:
     return new CSecurityStack(cc, secTypeTLSVnc,
-                              "TLS with VNCAuth",
                               new CSecurityTLS(cc, true),
                               new CSecurityVncAuth(cc));
   case secTypeTLSPlain:
     return new CSecurityStack(cc, secTypeTLSPlain,
-                              "TLS with Username/Password",
                               new CSecurityTLS(cc, true),
                               new CSecurityPlain(cc));
   case secTypeX509None:
     return new CSecurityStack(cc, secTypeX509None,
-                              "X509 with no password",
                               new CSecurityTLS(cc, false));
   case secTypeX509Vnc:
     return new CSecurityStack(cc, secTypeX509Vnc,
-                              "X509 with VNCAuth",
                               new CSecurityTLS(cc, false),
                               new CSecurityVncAuth(cc));
   case secTypeX509Plain:
     return new CSecurityStack(cc, secTypeX509Plain,
-                              "X509 with Username/Password",
                               new CSecurityTLS(cc, false),
                               new CSecurityPlain(cc));
+#endif
+#ifdef HAVE_NETTLE
+  case secTypeRA2:
+    return new CSecurityRSAAES(cc, secTypeRA2, 128, true);
+  case secTypeRA2ne:
+    return new CSecurityRSAAES(cc, secTypeRA2ne, 128, false);
+  case secTypeRA256:
+    return new CSecurityRSAAES(cc, secTypeRA256, 256, true);
+  case secTypeRAne256:
+    return new CSecurityRSAAES(cc, secTypeRAne256, 256, false);
 #endif
   }
 
