@@ -37,7 +37,7 @@
 namespace rfb {
   class VNCServerST;
 
-  class VNCSConnectionST : public SConnection,
+  class VNCSConnectionST : private SConnection,
                            public Timer::Callback {
   public:
     VNCSConnectionST(VNCServerST* server_, network::Socket* s, bool reverse);
@@ -48,6 +48,8 @@ namespace rfb {
     virtual bool accessCheck(AccessRights ar) const;
     virtual void close(const char* reason);
 
+    using SConnection::authenticated;
+
     // Methods called from VNCServerST.  None of these methods ever knowingly
     // throw an exception.
 
@@ -91,6 +93,11 @@ namespace rfb {
     // cursor.
     void renderedCursorChange();
 
+    // cursorPositionChange() is called whenever the cursor has changed position by
+    // the server.  If the client supports being informed about these changes then
+    // it will arrange for the new cursor position to be sent to the client.
+    void cursorPositionChange();
+
     // needRenderedCursor() returns true if this client needs the server-side
     // rendered cursor.  This may be because it does not support local cursor
     // or because the current cursor position has not been set by this client.
@@ -105,13 +112,14 @@ namespace rfb {
       updates.add_copied(dest, delta);
     }
 
+    const char* getPeerEndpoint() const {return peerEndpoint.buf;}
+
   private:
     // SConnection callbacks
 
     // These methods are invoked as callbacks from processMsg()
 
     virtual void authSuccess();
-    virtual void authFailure(const char* reason);
     virtual void queryConnection(const char* userName);
     virtual void clientInit(bool shared);
     virtual void setPixelFormat(const PixelFormat& pf);
@@ -152,9 +160,9 @@ namespace rfb {
 
     void screenLayoutChange(rdr::U16 reason);
     void setCursor();
+    void setCursorPos();
     void setDesktopName(const char *name);
     void setLEDState(unsigned int state);
-    void setSocketTimeouts();
 
   private:
     network::Socket* sock;
@@ -189,9 +197,6 @@ namespace rfb {
     Point pointerEventPos;
     bool clientHasCursor;
 
-    Timer authFailureTimer;
-    CharArray authFailureMsg;
-
     CharArray closeReason;
   };
 }
