@@ -48,6 +48,10 @@
 // otherwise blacklisted connections might be "forgotten".
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <stdlib.h>
 
@@ -137,7 +141,9 @@ void VNCServerST::addSocket(network::Socket* sock, bool outgoing)
       // Shortest possible way to tell a client it is not welcome
       os.writeBytes("RFB 003.003\n", 12);
       os.writeU32(0);
-      os.writeString("Too many security failures");
+      const char* reason = "Too many security failures";
+      os.writeU32(strlen(reason));
+      os.writeBytes(reason, strlen(reason));
       os.flush();
     } catch (rdr::Exception&) {
     }
@@ -169,21 +175,16 @@ void VNCServerST::removeSocket(network::Socket* sock) {
       if (pointerClient == *ci)
         pointerClient = NULL;
       if (clipboardClient == *ci)
-        clipboardClient = NULL;
+        handleClipboardAnnounce(*ci, false);
       clipboardRequestors.remove(*ci);
 
-      // Adjust the exit timers
-      connectTimer.stop();
-      if (rfb::Server::maxDisconnectionTime && clients.empty())
-        disconnectTimer.start(secsToMillis(rfb::Server::maxDisconnectionTime));
+      CharArray name(strDup((*ci)->getPeerEndpoint()));
 
       // - Delete the per-Socket resources
       delete *ci;
 
       clients.remove(*ci);
 
-      CharArray name;
-      name.buf = sock->getPeerEndpoint();
       connectionsLog.status("closed: %s", name.buf);
 
       // - Check that the desktop object is still required
@@ -193,6 +194,11 @@ void VNCServerST::removeSocket(network::Socket* sock) {
       if (comparer)
         comparer->logStats();
 
+      // Adjust the exit timers
+      connectTimer.stop();
+      if (rfb::Server::maxDisconnectionTime && clients.empty())
+        disconnectTimer.start(secsToMillis(rfb::Server::maxDisconnectionTime));
+
       return;
     }
   }
@@ -336,24 +342,23 @@ void VNCServerST::setScreenLayout(const ScreenSet& layout)
 
 void VNCServerST::requestClipboard()
 {
-  if (clipboardClient == NULL)
+  if (clipboardClient == NULL) {
+    slog.debug("Got request for client clipboard but no client currently owns the clipboard");
     return;
+  }
 
-  clipboardClient->requestClipboard();
+  clipboardClient->requestClipboardOrClose();
 }
 
 void VNCServerST::announceClipboard(bool available)
 {
   std::list<VNCSConnectionST*>::iterator ci, ci_next;
 
-  if (available)
-    clipboardClient = NULL;
-
   clipboardRequestors.clear();
 
   for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
     ci_next = ci; ci_next++;
-    (*ci)->announceClipboard(available);
+    (*ci)->announceClipboardOrClose(available);
   }
 }
 
@@ -367,7 +372,7 @@ void VNCServerST::sendClipboardData(const char* data)
   for (ci = clipboardRequestors.begin();
        ci != clipboardRequestors.end(); ci = ci_next) {
     ci_next = ci; ci_next++;
-    (*ci)->sendClipboardData(data);
+    (*ci)->sendClipboardDataOrClose(data);
   }
 
   clipboardRequestors.clear();
@@ -427,14 +432,17 @@ void VNCServerST::setCursor(int width, int height, const Point& newHotspot,
   }
 }
 
-void VNCServerST::setCursorPos(const Point& pos)
+void VNCServerST::setCursorPos(const Point& pos, bool warped)
 {
   if (!cursorPos.equals(pos)) {
     cursorPos = pos;
     renderedCursorInvalid = true;
     std::list<VNCSConnectionST*>::iterator ci;
-    for (ci = clients.begin(); ci != clients.end(); ci++)
+    for (ci = clients.begin(); ci != clients.end(); ci++) {
       (*ci)->renderedCursorChange();
+      if (warped)
+        (*ci)->cursorPositionChange();
+    }
   }
 }
 
@@ -515,8 +523,10 @@ void VNCServerST::handleClipboardAnnounce(VNCSConnectionST* client,
 void VNCServerST::handleClipboardData(VNCSConnectionST* client,
                                       const char* data)
 {
-  if (client != clipboardClient)
+  if (client != clipboardClient) {
+    slog.debug("Ignoring unexpected clipboard data");
     return;
+  }
   desktop->handleClipboardData(data);
 }
 
@@ -527,9 +537,18 @@ unsigned int VNCServerST::setDesktopSize(VNCSConnectionST* requester,
   unsigned int result;
   std::list<VNCSConnectionST*>::iterator ci, ci_next;
 
+  // We can't handle a framebuffer larger than this, so don't let a
+  // client set one (see PixelBuffer.cxx)
+  if ((fb_width > 16384) || (fb_height > 16384)) {
+    slog.error("Rejecting too large framebuffer resize request");
+    return resultProhibited;
+  }
+
   // Don't bother the desktop with an invalid configuration
-  if (!layout.validate(fb_width, fb_height))
+  if (!layout.validate(fb_width, fb_height)) {
+    slog.error("Invalid screen layout requested by client");
     return resultInvalid;
+  }
 
   // FIXME: the desktop will call back to VNCServerST and an extra set
   // of ExtendedDesktopSize messages will be sent. This is okay
@@ -594,7 +613,7 @@ SConnection* VNCServerST::getConnection(network::Socket* sock) {
   std::list<VNCSConnectionST*>::iterator ci;
   for (ci = clients.begin(); ci != clients.end(); ci++) {
     if ((*ci)->getSock() == sock)
-      return *ci;
+      return (SConnection*)*ci;
   }
   return 0;
 }
