@@ -1,4 +1,4 @@
-/* Copyright 2011 Pierre Ossman <ossman@cendio.se> for Cendio AB
+/* Copyright 2011-2021 Pierre Ossman <ossman@cendio.se> for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,18 +20,20 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdlib.h>
-
 #include <list>
 
 #include <rdr/types.h>
 #include <rfb/encodings.h>
 
-#ifdef HAVE_GNUTLS
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
 #include <rfb/Security.h>
 #include <rfb/SecurityClient.h>
+#ifdef HAVE_GNUTLS
 #include <rfb/CSecurityTLS.h>
 #endif
+#endif
 
 #include "OptionsDialog.h"
 #include "fltk_layout.h"
@@ -40,6 +42,7 @@
 #include "parameters.h"
 #include "MonitorArrangement.h"
 
+#include <FL/Fl.H>
 #include <FL/Fl_Tabs.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Check_Button.H>
@@ -54,8 +57,10 @@ using namespace rfb;
 
 std::map<OptionsCallback*, void*> OptionsDialog::callbacks;
 
+static std::set<OptionsDialog *> instances;
+
 OptionsDialog::OptionsDialog()
-  : Fl_Window(450, 450, _("VNC Viewer: Connection Options"))
+  : Fl_Window(450, 460, _("VNC Viewer: Connection Options"))
 {
   int x, y;
   Fl_Button *button;
@@ -72,7 +77,7 @@ OptionsDialog::OptionsDialog()
     createCompressionPage(tx, ty, tw, th);
     createSecurityPage(tx, ty, tw, th);
     createInputPage(tx, ty, tw, th);
-    createScreenPage(tx, ty, tw, th);
+    createDisplayPage(tx, ty, tw, th);
     createMiscPage(tx, ty, tw, th);
   }
 
@@ -92,11 +97,19 @@ OptionsDialog::OptionsDialog()
   callback(this->handleCancel, this);
 
   set_modal();
+
+  if (instances.size() == 0)
+    Fl::add_handler(fltk_event_handler);
+  instances.insert(this);
 }
 
 
 OptionsDialog::~OptionsDialog()
 {
+  instances.erase(this);
+
+  if (instances.size() == 0)
+    Fl::remove_handler(fltk_event_handler);
 }
 
 
@@ -153,6 +166,11 @@ void OptionsDialog::loadOptions(void)
   case encodingHextile:
     hextileButton->setonly();
     break;
+#ifdef HAVE_H264
+  case encodingH264:
+    h264Button->setonly();
+    break;
+#endif
   case encodingRaw:
     rawButton->setonly();
     break;
@@ -187,7 +205,7 @@ void OptionsDialog::loadOptions(void)
   handleCompression(compressionCheckbox, this);
   handleJpeg(jpegCheckbox, this);
 
-#ifdef HAVE_GNUTLS
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
   /* Security */
   Security security(SecurityClient::secTypes);
 
@@ -198,8 +216,13 @@ void OptionsDialog::loadOptions(void)
    list<U32>::iterator iterExt;
 
   encNoneCheckbox->value(false);
+#ifdef HAVE_GNUTLS
   encTLSCheckbox->value(false);
   encX509Checkbox->value(false);
+#endif
+#ifdef HAVE_NETTLE
+  encRSAAESCheckbox->value(false);
+#endif
 
   authNoneCheckbox->value(false);
   authVncCheckbox->value(false);
@@ -226,6 +249,7 @@ void OptionsDialog::loadOptions(void)
       encNoneCheckbox->value(true);
       authPlainCheckbox->value(true);
       break;
+#ifdef HAVE_GNUTLS
     case secTypeTLSNone:
       encTLSCheckbox->value(true);
       authNoneCheckbox->value(true);
@@ -250,13 +274,27 @@ void OptionsDialog::loadOptions(void)
       encX509Checkbox->value(true);
       authPlainCheckbox->value(true);
       break;
+#endif
+#ifdef HAVE_NETTLE
+    case secTypeRA2:
+    case secTypeRA256:
+      encRSAAESCheckbox->value(true);
+    case secTypeRA2ne:
+    case secTypeRAne256:
+      authVncCheckbox->value(true);
+      authPlainCheckbox->value(true);
+      break;
+#endif
+    
     }
   }
 
+#ifdef HAVE_GNUTLS
   caInput->value(CSecurityTLS::X509CA);
   crlInput->value(CSecurityTLS::X509CRL);
 
   handleX509(encX509Checkbox, this);
+#endif
 #endif
 
   /* Input */
@@ -281,35 +319,21 @@ void OptionsDialog::loadOptions(void)
     if (!strcmp(getMenuKeySymbols()[i].name, menuKeyBuf))
       menuKeyChoice->value(i + 1);
 
-  /* Screen */
-  int width, height;
-
-  if (sscanf((const char*)desktopSize, "%dx%d", &width, &height) != 2) {
-    desktopSizeCheckbox->value(false);
-    desktopWidthInput->value("1024");
-    desktopHeightInput->value("768");
+  /* Display */
+  if (!fullScreen) {
+    windowedButton->setonly();
   } else {
-    char buf[32];
-    desktopSizeCheckbox->value(true);
-    snprintf(buf, sizeof(buf), "%d", width);
-    desktopWidthInput->value(buf);
-    snprintf(buf, sizeof(buf), "%d", height);
-    desktopHeightInput->value(buf);
-  }
-  remoteResizeCheckbox->value(remoteResize);
-  fullScreenCheckbox->value(fullScreen);
-
-  if (!strcasecmp(fullScreenMode, "all")) {
-    allMonitorsButton->setonly();
-  } else if (!strcasecmp(fullScreenMode, "selected")) {
-    selectedMonitorsButton->setonly();
-  } else {
-    currentMonitorButton->setonly();
+    if (!strcasecmp(fullScreenMode, "all")) {
+      allMonitorsButton->setonly();
+    } else if (!strcasecmp(fullScreenMode, "selected")) {
+      selectedMonitorsButton->setonly();
+    } else {
+      currentMonitorButton->setonly();
+    }
   }
 
   monitorArrangement->set(fullScreenSelectedMonitors.getParam());
 
-  handleDesktopSize(desktopSizeCheckbox, this);
   handleFullScreenMode(selectedMonitorsButton, this);
 
   /* Misc. */
@@ -330,6 +354,10 @@ void OptionsDialog::storeOptions(void)
     preferredEncoding.setParam(encodingName(encodingZRLE));
   else if (hextileButton->value())
     preferredEncoding.setParam(encodingName(encodingHextile));
+#ifdef HAVE_H264
+  else if (h264Button->value())
+    preferredEncoding.setParam(encodingName(encodingH264));
+#endif
   else if (rawButton->value())
     preferredEncoding.setParam(encodingName(encodingRaw));
 
@@ -346,7 +374,7 @@ void OptionsDialog::storeOptions(void)
   compressLevel.setParam(atoi(compressionInput->value()));
   qualityLevel.setParam(atoi(jpegInput->value()));
 
-#ifdef HAVE_GNUTLS
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
   /* Security */
   Security security;
 
@@ -354,12 +382,22 @@ void OptionsDialog::storeOptions(void)
   if (encNoneCheckbox->value()) {
     if (authNoneCheckbox->value())
       security.EnableSecType(secTypeNone);
-    if (authVncCheckbox->value())
+    if (authVncCheckbox->value()) {
       security.EnableSecType(secTypeVncAuth);
-    if (authPlainCheckbox->value())
+#ifdef HAVE_NETTLE
+      security.EnableSecType(secTypeRAne256);
+#endif
+    }
+    if (authPlainCheckbox->value()) {
       security.EnableSecType(secTypePlain);
+#ifdef HAVE_NETTLE
+      security.EnableSecType(secTypeRA2ne);
+      security.EnableSecType(secTypeRAne256);
+#endif
+    }
   }
 
+#ifdef HAVE_GNUTLS
   /* Process security types which use TLS encryption */
   if (encTLSCheckbox->value()) {
     if (authNoneCheckbox->value())
@@ -380,12 +418,19 @@ void OptionsDialog::storeOptions(void)
       security.EnableSecType(secTypeX509Plain);
   }
 
-  SecurityClient::secTypes.setParam(security.ToString());
-
   CSecurityTLS::X509CA.setParam(caInput->value());
   CSecurityTLS::X509CRL.setParam(crlInput->value());
 #endif
 
+#ifdef HAVE_NETTLE
+  if (encRSAAESCheckbox->value()) {
+    security.EnableSecType(secTypeRA2);
+    security.EnableSecType(secTypeRA256);
+  }
+#endif
+  SecurityClient::secTypes.setParam(security.ToString());
+#endif
+
   /* Input */
   viewOnly.setParam(viewOnlyCheckbox->value());
   emulateMiddleButton.setParam(emulateMBCheckbox->value());
@@ -405,27 +450,19 @@ void OptionsDialog::storeOptions(void)
     menuKey.setParam(menuKeyChoice->text());
   }
 
-  /* Screen */
-  int width, height;
-
-  if (desktopSizeCheckbox->value() &&
-      (sscanf(desktopWidthInput->value(), "%d", &width) == 1) &&
-      (sscanf(desktopHeightInput->value(), "%d", &height) == 1)) {
-    char buf[64];
-    snprintf(buf, sizeof(buf), "%dx%d", width, height);
-    desktopSize.setParam(buf);
+  /* Display */
+  if (windowedButton->value()) {
+    fullScreen.setParam(false);
   } else {
-    desktopSize.setParam("");
-  }
-  remoteResize.setParam(remoteResizeCheckbox->value());
-  fullScreen.setParam(fullScreenCheckbox->value());
+    fullScreen.setParam(true);
 
-  if (allMonitorsButton->value()) {
-    fullScreenMode.setParam("All");
-  } else if (selectedMonitorsButton->value()) {
-    fullScreenMode.setParam("Selected");
-  } else {
-    fullScreenMode.setParam("Current");
+    if (allMonitorsButton->value()) {
+      fullScreenMode.setParam("All");
+    } else if (selectedMonitorsButton->value()) {
+      fullScreenMode.setParam("Selected");
+    } else {
+      fullScreenMode.setParam("Current");
+    }
   }
 
   fullScreenSelectedMonitors.setParam(monitorArrangement->get());
@@ -447,6 +484,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
   Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Compression"));
 
   int orig_tx, orig_ty;
+  int col1_ty, col2_ty;
   int half_width, full_width;
   int height;
 
@@ -471,6 +509,9 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
   /* VNC encoding box */
   ty += GROUP_LABEL_OFFSET;
   height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 3 + RADIO_HEIGHT * 4;
+#ifdef HAVE_H264
+  height += TIGHT_MARGIN + RADIO_HEIGHT;
+#endif
   encodingGroup = new Fl_Group(tx, ty, half_width, height,
                                 _("Preferred encoding"));
   encodingGroup->box(FL_ENGRAVED_BOX);
@@ -501,6 +542,15 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
     hextileButton->type(FL_RADIO_BUTTON);
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
 
+#ifdef HAVE_H264
+    h264Button = new Fl_Round_Button(LBLRIGHT(tx, ty,
+                                             RADIO_MIN_WIDTH,
+                                             RADIO_HEIGHT,
+                                             "H.264"));
+    h264Button->type(FL_RADIO_BUTTON);
+    ty += RADIO_HEIGHT + TIGHT_MARGIN;
+#endif
+
     rawButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                              RADIO_MIN_WIDTH,
                                              RADIO_HEIGHT,
@@ -512,6 +562,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
   ty += GROUP_MARGIN - TIGHT_MARGIN;
 
   encodingGroup->end();
+  col1_ty = ty;
 
   /* Second column */
   tx = orig_tx + half_width + INNER_MARGIN;
@@ -560,10 +611,11 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
   ty += GROUP_MARGIN - TIGHT_MARGIN;
 
   colorlevelGroup->end();
+  col2_ty = ty;
 
   /* Back to normal */
   tx = orig_tx;
-  ty += INNER_MARGIN;
+  ty = (col1_ty > col2_ty ? col1_ty : col2_ty) + INNER_MARGIN;
 
   /* Checkboxes */
   compressionCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
@@ -598,7 +650,7 @@ void OptionsDialog::createCompressionPage(int tx, int ty, int tw, int th)
 
 void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 {
-#ifdef HAVE_GNUTLS
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
   Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Security"));
 
   int orig_tx;
@@ -613,7 +665,14 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
 
   /* Encryption */
   ty += GROUP_LABEL_OFFSET;
+
+#if defined(HAVE_GNUTLS) && defined(HAVE_NETTLE)
+  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 5 + CHECK_HEIGHT * 4 + (INPUT_LABEL_OFFSET + INPUT_HEIGHT) * 2;
+#elif defined(HAVE_GNUTLS)
   height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 4 + CHECK_HEIGHT * 3 + (INPUT_LABEL_OFFSET + INPUT_HEIGHT) * 2;
+#elif defined(HAVE_NETTLE)
+  height = GROUP_MARGIN * 2 + TIGHT_MARGIN * 1 + CHECK_HEIGHT * 2;
+#endif
   encryptionGroup = new Fl_Group(tx, ty, width, height, _("Encryption"));
   encryptionGroup->box(FL_ENGRAVED_BOX);
   encryptionGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
@@ -628,6 +687,7 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
                                                    _("None")));
     ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
+#ifdef HAVE_GNUTLS
     encTLSCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                   CHECK_MIN_WIDTH,
                                                   CHECK_HEIGHT,
@@ -654,6 +714,15 @@ void OptionsDialog::createSecurityPage(int tx, int ty, int tw, int th)
                             _("Path to X509 CRL file"));
     crlInput->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
     ty += INPUT_HEIGHT + TIGHT_MARGIN;
+#endif
+#ifdef HAVE_NETTLE
+    encRSAAESCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+                                                     CHECK_MIN_WIDTH,
+                                                     CHECK_HEIGHT,
+                                                     "RSA-AES"));
+    encRSAAESCheckbox->callback(handleRSAAES, this);
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+#endif
   }
 
   ty += GROUP_MARGIN - TIGHT_MARGIN;
@@ -711,118 +780,188 @@ void OptionsDialog::createInputPage(int tx, int ty, int tw, int th)
 {
   Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Input"));
 
+  int orig_tx;
+  int width;
+
   tx += OUTER_MARGIN;
   ty += OUTER_MARGIN;
 
+  width = tw - OUTER_MARGIN * 2;
+
   viewOnlyCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                   CHECK_MIN_WIDTH,
                                                   CHECK_HEIGHT,
                                                   _("View only (ignore mouse and keyboard)")));
   ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
-  emulateMBCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                   CHECK_MIN_WIDTH,
-                                                   CHECK_HEIGHT,
-                                                   _("Emulate middle mouse button")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
+  orig_tx = tx;
 
-  acceptClipboardCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                         CHECK_MIN_WIDTH,
-                                                         CHECK_HEIGHT,
-                                                         _("Accept clipboard from server")));
-  acceptClipboardCheckbox->callback(handleClipboard, this);
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
+  /* Mouse */
+  ty += GROUP_LABEL_OFFSET;
+  mouseGroup = new Fl_Group(tx, ty, width, 0, _("Mouse"));
+  mouseGroup->box(FL_ENGRAVED_BOX);
+  mouseGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
 
-#if !defined(WIN32) && !defined(__APPLE__)
-  setPrimaryCheckbox = new Fl_Check_Button(LBLRIGHT(tx + INDENT, ty,
-                                                    CHECK_MIN_WIDTH,
-                                                    CHECK_HEIGHT,
-                                                    _("Also set primary selection")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
-#endif
+  /* Needed for final resize to work sanely */
+  mouseGroup->resizable(NULL);
 
-  sendClipboardCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                       CHECK_MIN_WIDTH,
-                                                       CHECK_HEIGHT,
-                                                       _("Send clipboard to server")));
-  sendClipboardCheckbox->callback(handleClipboard, this);
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
+  {
+    tx += GROUP_MARGIN;
+    ty += GROUP_MARGIN;
 
-#if !defined(WIN32) && !defined(__APPLE__)
-  sendPrimaryCheckbox = new Fl_Check_Button(LBLRIGHT(tx + INDENT, ty,
+    emulateMBCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                      CHECK_MIN_WIDTH,
                                                      CHECK_HEIGHT,
-                                                     _("Send primary selection as clipboard")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
-#endif
+                                                     _("Emulate middle mouse button")));
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
-  systemKeysCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+    dotCursorCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
                                                     CHECK_MIN_WIDTH,
                                                     CHECK_HEIGHT,
-                                                    _("Pass system keys directly to server (full screen)")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
+                                                    _("Show dot when no cursor")));
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+  }
+  ty += GROUP_MARGIN - TIGHT_MARGIN;
 
-  menuKeyChoice = new Fl_Choice(LBLLEFT(tx, ty, 150, CHOICE_HEIGHT, _("Menu key")));
+  mouseGroup->end();
+  mouseGroup->size(mouseGroup->w(), ty - mouseGroup->y());
 
-  fltk_menu_add(menuKeyChoice, _("None"), 0, NULL, (void*)0, FL_MENU_DIVIDER);
-  for (int i = 0; i < getMenuKeySymbolCount(); i++)
-    fltk_menu_add(menuKeyChoice, getMenuKeySymbols()[i].name, 0, NULL, 0, 0);
+  /* Back to normal */
+  tx = orig_tx;
+  ty += INNER_MARGIN;
 
-  ty += CHOICE_HEIGHT + TIGHT_MARGIN;
+  /* Keyboard */
+  ty += GROUP_LABEL_OFFSET;
+  keyboardGroup = new Fl_Group(tx, ty, width, 0, _("Keyboard"));
+  keyboardGroup->box(FL_ENGRAVED_BOX);
+  keyboardGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
+
+  /* Needed for final resize to work sanely */
+  keyboardGroup->resizable(NULL);
+
+  {
+    tx += GROUP_MARGIN;
+    ty += GROUP_MARGIN;
+
+    systemKeysCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+                                                      CHECK_MIN_WIDTH,
+                                                      CHECK_HEIGHT,
+                                                      _("Pass system keys directly to server (full screen)")));
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+
+    menuKeyChoice = new Fl_Choice(LBLLEFT(tx, ty, 150, CHOICE_HEIGHT, _("Menu key")));
+
+    fltk_menu_add(menuKeyChoice, _("None"), 0, NULL, (void*)0, FL_MENU_DIVIDER);
+    for (int i = 0; i < getMenuKeySymbolCount(); i++)
+      fltk_menu_add(menuKeyChoice, getMenuKeySymbols()[i].name, 0, NULL, 0, 0);
+
+    ty += CHOICE_HEIGHT + TIGHT_MARGIN;
+  }
+  ty += GROUP_MARGIN - TIGHT_MARGIN;
+
+  keyboardGroup->end();
+  keyboardGroup->size(keyboardGroup->w(), ty - keyboardGroup->y());
+
+  /* Back to normal */
+  tx = orig_tx;
+  ty += INNER_MARGIN;
+
+  /* Clipboard */
+  ty += GROUP_LABEL_OFFSET;
+  clipboardGroup = new Fl_Group(tx, ty, width, 0, _("Clipboard"));
+  clipboardGroup->box(FL_ENGRAVED_BOX);
+  clipboardGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
+
+  /* Needed for final resize to work sanely */
+  clipboardGroup->resizable(NULL);
+
+  {
+    tx += GROUP_MARGIN;
+    ty += GROUP_MARGIN;
+
+    acceptClipboardCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+                                                           CHECK_MIN_WIDTH,
+                                                           CHECK_HEIGHT,
+                                                           _("Accept clipboard from server")));
+    acceptClipboardCheckbox->callback(handleClipboard, this);
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+
+#if !defined(WIN32) && !defined(__APPLE__)
+    setPrimaryCheckbox = new Fl_Check_Button(LBLRIGHT(tx + INDENT, ty,
+                                                      CHECK_MIN_WIDTH,
+                                                      CHECK_HEIGHT,
+                                                      _("Also set primary selection")));
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+#endif
+
+    sendClipboardCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
+                                                         CHECK_MIN_WIDTH,
+                                                         CHECK_HEIGHT,
+                                                         _("Send clipboard to server")));
+    sendClipboardCheckbox->callback(handleClipboard, this);
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+
+#if !defined(WIN32) && !defined(__APPLE__)
+    sendPrimaryCheckbox = new Fl_Check_Button(LBLRIGHT(tx + INDENT, ty,
+                                                       CHECK_MIN_WIDTH,
+                                                       CHECK_HEIGHT,
+                                                       _("Send primary selection as clipboard")));
+    ty += CHECK_HEIGHT + TIGHT_MARGIN;
+#endif
+  }
+  ty += GROUP_MARGIN - TIGHT_MARGIN;
+
+  clipboardGroup->end();
+  clipboardGroup->size(clipboardGroup->w(), ty - clipboardGroup->y());
+
+  /* Back to normal */
+  tx = orig_tx;
+  ty += INNER_MARGIN;
 
   group->end();
 }
 
 
-void OptionsDialog::createScreenPage(int tx, int ty, int tw, int th)
+void OptionsDialog::createDisplayPage(int tx, int ty, int tw, int th)
 {
-  int x;
-  int width, height;
+  Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Display"));
 
-  Fl_Group *group = new Fl_Group(tx, ty, tw, th, _("Screen"));
+  int orig_tx;
+  int width;
 
   tx += OUTER_MARGIN;
   ty += OUTER_MARGIN;
 
-  desktopSizeCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                     CHECK_MIN_WIDTH,
-                                                     CHECK_HEIGHT,
-                                                     _("Resize remote session on connect")));
-  desktopSizeCheckbox->callback(handleDesktopSize, this);
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
-
-  desktopWidthInput = new Fl_Int_Input(tx + INDENT, ty, 50, INPUT_HEIGHT);
-  x = desktopWidthInput->x() + desktopWidthInput->w() + \
-      gui_str_len("x") + 3 * 2;
-  desktopHeightInput = new Fl_Int_Input(x, ty, 50, INPUT_HEIGHT, "x");
-  ty += INPUT_HEIGHT + TIGHT_MARGIN;
-
-  remoteResizeCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                      CHECK_MIN_WIDTH,
-                                                      CHECK_HEIGHT,
-                                                      _("Resize remote session to the local window")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
-
-  fullScreenCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                  CHECK_MIN_WIDTH,
-                                                  CHECK_HEIGHT,
-                                                  _("Enable full-screen")));
-  ty += CHECK_HEIGHT + INNER_MARGIN;
-
   width = tw - OUTER_MARGIN * 2;
-  height = th - ty + OUTER_MARGIN * 3;
-    Fl_Group *fullScreenModeGroup = new Fl_Group(tx,
-                                     ty,
-                                     width,
-                                     height);
+
+  orig_tx = tx;
+
+  /* Display mode */
+  ty += GROUP_LABEL_OFFSET;
+  displayModeGroup = new Fl_Group(tx, ty, width, 0, _("Display mode"));
+  displayModeGroup->box(FL_ENGRAVED_BOX);
+  displayModeGroup->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
+
+  /* Needed for final resize to work sanely */
+  displayModeGroup->resizable(NULL);
 
   {
-    tx += INDENT;
+    tx += GROUP_MARGIN;
+    ty += GROUP_MARGIN;
+    width -= GROUP_MARGIN * 2;
+
+    windowedButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
+                                                  RADIO_MIN_WIDTH,
+                                                  RADIO_HEIGHT,
+                                                  _("Windowed")));
+    windowedButton->type(FL_RADIO_BUTTON);
+    windowedButton->callback(handleFullScreenMode, this);
+    ty += RADIO_HEIGHT + TIGHT_MARGIN;
 
     currentMonitorButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                                         RADIO_MIN_WIDTH,
                                                         RADIO_HEIGHT,
-                                                        _("Use current monitor")));
+                                                        _("Full screen on current monitor")));
     currentMonitorButton->type(FL_RADIO_BUTTON);
     currentMonitorButton->callback(handleFullScreenMode, this);
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
@@ -830,7 +969,7 @@ void OptionsDialog::createScreenPage(int tx, int ty, int tw, int th)
     allMonitorsButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                             RADIO_MIN_WIDTH,
                                             RADIO_HEIGHT,
-                                            _("Use all monitors")));
+                                            _("Full screen on all monitors")));
     allMonitorsButton->type(FL_RADIO_BUTTON);
     allMonitorsButton->callback(handleFullScreenMode, this);
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
@@ -838,26 +977,27 @@ void OptionsDialog::createScreenPage(int tx, int ty, int tw, int th)
     selectedMonitorsButton = new Fl_Round_Button(LBLRIGHT(tx, ty,
                                                  RADIO_MIN_WIDTH,
                                                  RADIO_HEIGHT,
-                                                 _("Use selected monitor(s)")));
+                                                 _("Full screen on selected monitor(s)")));
     selectedMonitorsButton->type(FL_RADIO_BUTTON);
     selectedMonitorsButton->callback(handleFullScreenMode, this);
     ty += RADIO_HEIGHT + TIGHT_MARGIN;
 
-    int full_width = tw - OUTER_MARGIN * 2;
-    int margin_width = full_width - INDENT - INNER_MARGIN*2;
-    int full_height = th;
-    int margin_height = full_height - ty + INNER_MARGIN*3;
-
     monitorArrangement = new MonitorArrangement(
-                              tx + INDENT,
-                              ty,
-                              margin_width,
-                              margin_height);
-
-    ty += CHECK_HEIGHT + margin_height;
+                              tx + INDENT, ty,
+                              width - INDENT, 150);
+    ty += 150 + TIGHT_MARGIN;
   }
+  ty += GROUP_MARGIN - TIGHT_MARGIN;
+
+  displayModeGroup->end();
+  displayModeGroup->size(displayModeGroup->w(),
+                         ty - displayModeGroup->y());
+
+  /* Back to normal */
+  tx = orig_tx;
+  ty += INNER_MARGIN;
+  width = tw - OUTER_MARGIN * 2;
 
-  fullScreenModeGroup->end();
   group->end();
 }
 
@@ -881,12 +1021,6 @@ void OptionsDialog::createMiscPage(int tx, int ty, int tw, int th)
                                                   _("Ask to reconnect on connection errors")));
   ty += CHECK_HEIGHT + TIGHT_MARGIN;
 
-  dotCursorCheckbox = new Fl_Check_Button(LBLRIGHT(tx, ty,
-                                                  CHECK_MIN_WIDTH,
-                                                  CHECK_HEIGHT,
-                                                  _("Show dot when no cursor")));
-  ty += CHECK_HEIGHT + TIGHT_MARGIN;
-
   group->end();
 }
 
@@ -945,19 +1079,17 @@ void OptionsDialog::handleX509(Fl_Widget *widget, void *data)
 }
 
 
-void OptionsDialog::handleDesktopSize(Fl_Widget *widget, void *data)
+void OptionsDialog::handleRSAAES(Fl_Widget *widget, void *data)
 {
   OptionsDialog *dialog = (OptionsDialog*)data;
 
-  if (dialog->desktopSizeCheckbox->value()) {
-    dialog->desktopWidthInput->activate();
-    dialog->desktopHeightInput->activate();
-  } else {
-    dialog->desktopWidthInput->deactivate();
-    dialog->desktopHeightInput->deactivate();
+  if (dialog->encRSAAESCheckbox->value()) {
+    dialog->authVncCheckbox->value(true);
+    dialog->authPlainCheckbox->value(true);
   }
 }
 
+
 void OptionsDialog::handleClipboard(Fl_Widget *widget, void *data)
 {
 #if !defined(WIN32) && !defined(__APPLE__)
@@ -1001,3 +1133,28 @@ void OptionsDialog::handleOK(Fl_Widget *widget, void *data)
 
   dialog->storeOptions();
 }
+
+int OptionsDialog::fltk_event_handler(int event)
+{
+  std::set<OptionsDialog *>::iterator iter;
+
+  if (event != FL_SCREEN_CONFIGURATION_CHANGED)
+    return 0;
+
+  // Refresh monitor arrangement widget to match the parameter settings after
+  // screen configuration has changed. The MonitorArrangement index doesn't work
+  // the same way as the FLTK screen index.
+  for (iter = instances.begin(); iter != instances.end(); iter++)
+      Fl::add_timeout(0, handleScreenConfigTimeout, (*iter));
+
+  return 0;
+}
+
+void OptionsDialog::handleScreenConfigTimeout(void *data)
+{
+    OptionsDialog *self = (OptionsDialog *)data;
+
+    assert(self);
+
+    self->monitorArrangement->set(fullScreenSelectedMonitors.getParam());
+}
