@@ -20,9 +20,11 @@
 #define __SERVERDIALOG_H__
 
 #include <FL/Fl_Window.H>
+#include <string>
+#include <vector>
 
 class Fl_Widget;
-class Fl_Input;
+class Fl_Input_Choice;
 
 class ServerDialog : public Fl_Window {
 protected:
@@ -40,8 +42,15 @@ protected:
   static void handleCancel(Fl_Widget *widget, void *data);
   static void handleConnect(Fl_Widget *widget, void *data);
 
+private:
+  void loadServerHistory();
+  void saveServerHistory();
+  void updateUsedDir(const char* filename);
+
 protected:
-  Fl_Input *serverName;
+  Fl_Input_Choice *serverName;
+  std::vector<std::string> serverHistory;
+  char *usedDir;
 };
 
 #endif
