@@ -56,6 +56,7 @@
 #include <rfb/LogWriter.h>
 #include <rfb/Timer.h>
 #include <rfb/Exception.h>
+#include <rdr/Exception.h>
 #include <network/TcpSocket.h>
 #include <os/os.h>
 
@@ -80,7 +81,7 @@
 #include "win32.h"
 #endif
 
-rfb::LogWriter vlog("main");
+static rfb::LogWriter vlog("main");
 
 using namespace network;
 using namespace rfb;
@@ -103,12 +104,11 @@ static const char *about_text()
   // encodings, so we need to make sure we get a fresh string every
   // time.
   snprintf(buffer, sizeof(buffer),
-           _("TigerVNC Viewer %d-bit v%s\n"
+           _("TigerVNC Viewer v%s\n"
              "Built on: %s\n"
              "Copyright (C) 1999-%d TigerVNC Team and many others (see README.rst)\n"
              "See https://www.tigervnc.org for information on TigerVNC."),
-           (int)sizeof(size_t)*8, PACKAGE_VERSION,
-           BUILD_TIMESTAMP, 2021);
+           PACKAGE_VERSION, BUILD_TIMESTAMP, 2022);
 
   return buffer;
 }
@@ -157,6 +157,11 @@ void abort_connection(const char *error, ...)
   exitMainloop = true;
 }
 
+void abort_connection_with_unexpected_error(const rdr::Exception &e) {
+  abort_connection(_("An unexpected error occurred when communicating "
+                     "with the server:\n\n%s"), e.str());
+}
+
 void disconnect()
 {
   exitMainloop = true;
@@ -350,9 +355,9 @@ static void init_fltk()
 
   Fl_Window::default_icons(lg, sm);
 #elif ! defined(__APPLE__)
-  const int icon_sizes[] = {48, 32, 24, 16};
+  const int icon_sizes[] = {128, 64, 48, 32, 24, 22, 16};
 
-  Fl_PNG_Image *icons[4];
+  Fl_PNG_Image *icons[sizeof(icon_sizes)/sizeof(icon_sizes[0])];
   int count;
 
   count = 0;
@@ -365,14 +370,8 @@ static void init_fltk()
       sprintf(icon_path, "%s/icons/hicolor/%dx%d/apps/tigervnc.png",
               CMAKE_INSTALL_FULL_DATADIR, icon_sizes[i], icon_sizes[i]);
 
-#ifndef WIN32
       struct stat st;
       if (stat(icon_path, &st) != 0)
-#else
-      struct _stat st;
-      if (_stat(icon_path, &st) != 0)
-          return(false);
-#endif
         exists = false;
       else
         exists = true;
@@ -461,7 +460,7 @@ static void mkvnchomedir()
   } else {
     int result = mkdir(homeDir, 0755);
     if (result == -1 && errno != EEXIST)
-      vlog.error(_("Could not create VNC home directory: %s."), strerror(errno));
+      vlog.error(_("Could not create VNC home directory: %s"), strerror(errno));
     delete [] homeDir;
   }
 }
@@ -755,8 +754,8 @@ int main(int argc, char** argv)
   mkvnchomedir();
 
   CSecurity::upg = &dlg;
-#ifdef HAVE_GNUTLS
-  CSecurityTLS::msg = &dlg;
+#if defined(HAVE_GNUTLS) || defined(HAVE_NETTLE)
+  CSecurity::msg = &dlg;
 #endif
 
   Socket *sock = NULL;
